''
'' UCI-Monitor interface for Fidelity-EAG-Interface (COM port)
'' Optimized for Arena
'' V1.0
''
'' (C) 2012 by Achim Pietig <achim@pietig.com>
'' License:
''
'' Redistribution and use in source and binary forms, with or without
'' modification, are permitted provided that the following conditions
'' are met:
''
'' 1. Redistributions of source code must retain the above copyright
''    notice, this list of conditions and the following disclaimer.
'' 2. Redistributions in binary form must reproduce the above copyright
''    notice, this list of conditions and the following disclaimer in the
''    documentation and/or other materials provided with the distribution.
'' 3. The name of the author may not be used to endorse or promote products
''    derived from this software without specific prior written permission.
''
'' Changes to this license can be made only by the copyright author with
'' explicit written consent.
''
'' THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
'' IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
'' OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
'' IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
'' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
'' NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
'' DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
'' THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
'' (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
'' THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'
'   Main program
'

DIM As UShort n, i, black
DIM As String indata, dat, cdata, bestmove
DIM As String actCOM, wm, bm
Dim Shared nParams As UByte
ReDim Shared Param(1 To 2) As String
' Sets default command line parameters
actCOM = "COM1:"

' Checking for command line parameters
nParams = 0
While Command(nParams + 1) <> ""
    nParams = nParams + 1
Wend
If nParams > 0 Then
    ReDim Param(1 To nParams) As String
    For i = 1 To nParams
        Param(i) = Command(i)
    Next i
End If

' Help
If UCase(Param(1)) = "/?" Or UCase(Param(1)) = "/H" Then
    Print
    Print "EAG-Monitor for Fidelity chess computers"
    Print "(C) 2012 by Achim Pietig, Version 1.0"
    Print
    Print "Usage: EAG-Monitor [/?] Or [/h]"
    Print "       EAG-Monitor [COMx:]"
    Print "Call without parameters sets COM1:"
    Print
    Print "e.g.   EAG-Monitor             (COM1:)"
    Print "       EAG-Monitor COM2:       (COM2:)"
    End
End If

' Init program parameters
If nParams > 0 Then actCOM = UCase(Param(1))

' Opens related COM port with board specific values
' EAG cable supports TxD, RxD and RTS only. All other hardware specific handshakes are disabled.
Open Cons For Input As #1
If Open Com (actCOM + "600,N,8,1,CS0,DS0,CD0,BIN" For Binary As #2) <> 0 Then
    Print "Cannot initialise comport"
    Close
    End
End If

' Program loop
black = 0
Do
        ' Waits for an Arena UCI command
        Line Input #1, indata
        indata = Trim(indata)
        cdata = "" : dat = "" : bestmove = ""

        ' Answer to Arena in case of 'uci'
        If indata = "uci" Then
            Print "id name EAG-Monitor V1.0"
            Print "id author Achim Pietig"
            ' Typical values for chess computers with own book
            Print "option name OwnBook type check default true"
            Print "uciok"
        End If

        ' Answer to Arena in case of 'isready'
        If indata = "isready" Then
            Print "readyok"
        End If

        ' Action in case of 'go' command from Arena, response is 'bestmove' with the move from the board
        If Left(indata, 2) = "go" Then
            If black = 0 Then
                Do
                    bestmove = "" : cdata = ""
                    ' Read comlete move line for black and white
                    Do
                        dat = Input(1, #2)
                        ' Older computers use 7-bit-Ascii, this is corrected here
                        If Asc(dat) > 127 Then dat = Chr(Asc(dat) - 128)
                        ' Some computers send CR/LF others only LF, so CR is stripped off
                        ' CC Voice sends "-" between move coordinates, others don't
                        If dat <> Chr(&H0D) And dat <> "-" Then cdata = cdata + dat
                    Loop Until dat = Chr(&H0A)
                    ' Strip off LF
                    If Right(cdata, 1) = Chr(&H0A) Then cdata = Left(cdata, Len(cdata) - 1)
                    cdata = Trim(cdata)
                    
                    ' Identifiy moves
                    If Instr("0123456789", Left(cdata, 1)) > 0 And Instr("0123456789", Mid(cdata, 2, 1)) > 0 Then
                        ' Strip off move number
                        n = Instr(cdata, " ")
                        cdata = Trim(Mid(cdata, n + 1))
                        ' Eliminate double spaces
                        While Instr(cdata, "  ")
                            n = Instr(cdata, "  ")
                            cdata = Left(cdata, n - 1) + Mid(cdata, n + 1)
                        Wend
                        ' Separate moves for white and black
                        wm = LCase(Left(cdata, 4))
                        bm = LCase(Mid(cdata, 6, 4))
                        ' Set move for white
                        bestmove = wm
                        black = 1
                    Else
                        ' Other values are gives as info to Arena
                        If Len(cdata) > 1 Then Print "info string "; cdata
                    End If
                Loop Until Len(bestmove) > 2
                Print "bestmove "; bestmove
            Else
                ' Set move for black
                bestmove = bm
                black = 0
                Print "bestmove "; bestmove
            End If
        End If

' Action on Arena command 'quit'
Loop Until indata = "quit"

' Close open files and release COM port
Close
End
